/* Hercules Graphic Card (HGC), "Grafik Test"                           */

/* Die Routinen SetGraph und PSet sind so "eingestellt", dass schwarze  */
/* Punkte auf weissem Bildschirm gesetzt werden (siehe auch dort)       */

#include <mem.h>
#include <dos.h>

/* Register der HGC */

unsigned int IndexReg  = 0x03B4;
unsigned int DatenReg  = 0x03B5;
unsigned int ModusReg  = 0x03B8;
unsigned int KonfigReg = 0x03Bf;

/* Zeiger zum Bildschirmspeicher der HGC (4 Baenke) */

unsigned int HGCBank[] = {0xB000, 0xB200, 0xB400, 0xB600};

/* Funktions Definitionen */

void SetGraph(void);      /* Grafik-Modus setzen */
void PSet(int x, int y);  /* Punkt setzen        */
void SetText(void);       /* Textmodus setzen    */
void Pause(void);

/*                */
/* Haupt-Funktion */
/*                */

int main()
{
  SetGraph();

  /* Verschiedene Punkte auf dem Grafikbildschirm setzen:           */

  PSet(    0,    0);  /* ganz links oben                            */
  PSet(    1,    1);  /* "links oben" je 1 Pixel vom Rand entfernt  */
  PSet(720/2,348/2);  /* in der Bildmitte                           */
  PSet(719  ,  347);  /* ganz rechts unten                          */

  Pause();
  SetText();
  return(0);
}


/*                             */
/* PSet(x, y) Bildpunkt setzen */
/*                             */

void PSet( int x, int y)
{
 unsigned int bank;
 unsigned int pkt_addr;
 unsigned char far *screen;
 unsigned short pixel[]  = {128, 64, 32, 16, 8, 4, 2, 1};

 /* Weil direkt in den Bildschirmspeicher geschrieben wird, sollten */
 /* zu grosse oder kleine Werte obergangen werden, deshalb IF.....  */

 if ((x >= 0) && (x < 720) && (y >= 0) && (y < 348))
 {
   bank = y % 4;
   pkt_addr = x >> 3;
   pkt_addr += 90 * ((y-bank) >> 2);

   screen = MK_FP (HGCBank[bank], pkt_addr);
/*   screen[0] |= pixel[x % 8];        Punkt wird weiss   */
   screen[0] &= 255 - pixel[x % 8]; /* Punkt wird schwarz */
 };
}


/*                             */
/* HGC in Textmodus umschalten */
/*                             */

void SetText(void)
{ unsigned char i;
  unsigned int j;
  unsigned int far *screen;
  unsigned char TextModus[] = {0x61, 0x50, 0x52, 0x0f, 0x19, 0x06,
                               0x19, 0x19, 0x02, 0x0d, 0x0b, 0x0c,
                               0x00, 0x00, 0x00, 0x01};

  outportb (KonfigReg, 0x00);         /* Begrenzter Zugriff auf HGC        */
  outportb (ModusReg,  0x20);         /* Textmodus setzen und Bildschirm   */
                                      /* ausschalten                       */
  for (i=0 ; i < 16; i++)
  {  outportb (IndexReg,  i);         /* Register der HGC setzen           */
     outportb (DatenReg,  TextModus[i]);
  };
  screen = MK_FP (HGCBank[0], 0);     /* Text-Bildschirmspeicher loeschen  */
  for (j = 0 ; j < 0x2000; j++)       /* d.h. mit Leerzeichen und Attribut */
   screen[j]   = 0x0720;              /* normalweiss (0x07) follen.        */

  outportb (ModusReg,  0x28);         /* Bildschirm einschalten            */
}


/*                               */
/* HGC in Grafikmodus umschalten */
/*                               */

void SetGraph(void)
{ unsigned char i;
  unsigned int  j;
  unsigned int far *screen;
  unsigned char GrafikModus[] = {0x35, 0x2d, 0x2e, 0x07, 0x5b, 0x02,
                                 0x57, 0x57, 0x02, 0x03, 0x00, 0x00,
                                 0x00, 0x00, 0x00, 0x01};

  outportb (KonfigReg, 0x03);        /* Voller Zugriff auf HGC             */
  outportb (ModusReg,  0x02);        /* Grafikmodus setzen und Bildschirm  */
                                     /* ausschalten                        */
  for (i = 0; i < 16; i++)
  {  outportb (IndexReg,  i);        /* Register der HGC setzen            */
     outportb (DatenReg,  GrafikModus[i]);
  };

  for (j = 0 ; j < 0x2000; j++)
  {
    screen = MK_FP (HGCBank[0], 0);  /* Grafik-Bildschirmspeicher loeschen */
    screen[j]   = 0xFFFF;            /* d.h. mit 0x0000 (for schwarz)      */
    screen = MK_FP (HGCBank[1], 0);  /* bzw 0xFFFF for (weiss) follen.     */
    screen[j]   = 0xFFFF;
    screen = MK_FP (HGCBank[2], 0);
    screen[j]   = 0xFFFF;
    screen = MK_FP (HGCBank[3], 0);
    screen[j]   = 0x0;
  };
  outportb (ModusReg,  0x0A);        /* Bildschirm einschalten             */
}

void Pause(void)
{
  int c;
  c = getch();
  if( 0 == c ){
      c = getch();
  }
}
