/*
   Copyright (c) 1991 - 1993 Heinz W. Werntges.  All rights reserved.
   Distributed by Free Software Foundation, Inc.

This file is part of HP2xx.

HP2xx is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
HP2xx, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with HP2xx so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

/** to_pcl.c: PCL converter part of project "hp2xx"
 **
 ** 91/01/19  V 1.00  HWW  Reorganized
 ** 91/01/29  V 1.01  HWW  Tested on SUN
 ** 91/02/01  V 1.02  HWW  Deskjet specials acknowledged
 ** 91/02/15  V 1.03  HWW  VAX_C support added
 ** 91/02/20  V 1.04b HWW  x & y positioning: Now absolute!
 **			  Some VAX_C changes
 ** 91/06/09  V 1.05  HWW  New options added
 ** 91/10/15  V 1.06  HWW  ANSI_C
 ** 91/10/25  V 1.07  HWW  VAX: fopen() augmentations used, open() removed
 ** 92/05/17  V 1.07b HWW  Output to stdout if outfile == '-'
 ** 92/05/19  V 1.07c HWW  Abort if color mode
 ** 92/12/23  V 1.08a HWW  Color for Deskjet (beginning)
 **/

#include <stdio.h>
#include <stdlib.h>
#include "bresnham.h"
#include "hp2xx.h"




void	RowBuf_to_PCL_color (RowBuf *row, int nb, FILE *fd)
{
char	*p_c, *p_g, *p_y;

  p_y = (char *) row->buf;
  p_g = p_y + sizeof(DotBlock) * nb;
  p_c = p_g + sizeof(DotBlock) * nb;
  fprintf(fd,"\033*b%1dv", nb);
  fwrite (p_c, sizeof(DotBlock) * nb, 1, fd);
  fprintf(fd,"%1dv", nb);
  fwrite (p_g, sizeof(DotBlock) * nb, 1, fd);
  fprintf(fd,"%1dW", nb);
  fwrite (p_y, sizeof(DotBlock) * nb, 1, fd);
}




void	RowBuf_color_to_bw (RowBuf *row, int nb)
{
int	i;
char	*p_1, *p_2, *p_3;

  p_1 = (char *) row->buf;
  p_2 = p_1 + sizeof(DotBlock) * nb;
  p_3 = p_2 + sizeof(DotBlock) * nb;
  for (i=0; i < nb; i++)
	*p_1++ |= (*p_2++ | *p_3++);
}




void	PicBuf_to_PCL (PicBuf *picbuf, PAR *p)
{
FILE	*fd;
RowBuf	*row;
int	row_c, x, save_specials;

  if (!p->quiet)
	fprintf(stderr, "\nWriting PCL output\n");
  if (picbuf->depth > 1 && p->specials < 3)
	fprintf(stderr,
	    "\nWARNING: Monochrome output despite active colors selected!\n");

  save_specials = p->specials;
  if (picbuf->depth < p->specials)
  {
	fprintf(stderr,
	    "\nCannot convert to CMY(K) -- falling back to monochrome...\n");
	p->specials &= 1;
  }

  if (*p->outfile != '-')
  {
#ifdef VAX
	if ((fd = fopen(p->outfile, WRITE_BIN, "rfm=var","mrs=512")) == NULL)
	{
#else
	if ((fd = fopen(p->outfile, WRITE_BIN)) == NULL)
	{
#endif
		perror ("hp2xx -- opening output file");
		free_PicBuf (picbuf, p->swapfile);
		exit (ERROR);
	}
  }
  else	fd = stdout;

  if (p->init_p)
	init_printer (fd);

  start_graphmode (p, picbuf->nc, fd);

  /* Backward since highest index is lowest line on paper! */

  for (row_c = picbuf->nr - 1; row_c >= 0; row_c--)
  {
	if ((!p->quiet) && (row_c % 10 == 0))
		  /* For the impatients among us ...	*/
		putc('.',stderr);

	row = get_RowBuf(picbuf, row_c);
	switch (p->specials)
	{
	  case 4:	/* For future DJ 550 (CMYK) support)	*/
	  case 3:	/* DJ 500C (CMY)			*/
		/**
		 ** Convert all 3-bit color codes into CMY codes
		 ** Use internal row buffers to save mem.
		 ** NOTE: plot_RowBuf() is slow.
		 **/
		for (x=0; x < (picbuf->nb << 3); x++)
			switch (index_from_RowBuf(row, x, picbuf))
			{
			  case xxBackground:
				plot_RowBuf (row, x, picbuf, 0); break;
			  case xxForeground:
				plot_RowBuf (row, x, picbuf, 7); break;
			  case xxRed:
				plot_RowBuf (row, x, picbuf, 3); break;
			  case xxGreen:
				plot_RowBuf (row, x, picbuf, 5); break;
			  case xxBlue:
				plot_RowBuf (row, x, picbuf, 6); break;
			  case xxCyan:
				plot_RowBuf (row, x, picbuf, 4); break;
			  case xxMagenta:
				plot_RowBuf (row, x, picbuf, 2); break;
			  case xxYellow:
				plot_RowBuf (row, x, picbuf, 1); break;
			  default:
				plot_RowBuf (row, x, picbuf, 7); break;
			}
		RowBuf_to_PCL_color (row, picbuf->nb, fd);
		break;

	  default:
		if (picbuf->depth > 1)
			RowBuf_color_to_bw (row, picbuf->nb);
		RowBuf_to_PCL	    (row, picbuf->nb, fd);
		break;
	}
  }

  end_graphmode (fd);
  if (p->formfeed)
	putc (FF, fd);
  if (!p->quiet)
	fputc ('\n', stderr);
  if (fd != stdout)
	fclose (fd);
  p->specials = save_specials;
}




void	RowBuf_to_PCL (RowBuf *row, int nb, FILE *fd)
{
/**
 ** Currently, DotBlock=char, so there is no Hi/lo-endian problem,
 ** and printer output can be handled byte-by-byte.
 ** DO NOT change DotBlock without e.g. rewriting this whole routine!
 **/

  fprintf(fd,"\033*b%dW", nb);
/*  Following change keeps the VAX people happy:		*/
/*fwrite ((char *) row->buf, sizeof(DotBlock),     nb, fd);	*/
  fwrite ((char *) row->buf, sizeof(DotBlock) * nb, 1, fd);
}




void	init_printer (FILE *fd)
{
  fputc(ESC,fd); fputc('E',fd);	/* Esc-E */
}




void	start_graphmode (PAR *p, int width, FILE *fd)
{
/**
 ** X & Y offsets: Use "decipoints" as unit to stick to PCL level 3
 **		1 dpt = 0.1 pt = 1/720 in
 **/
  if (p->yoff != 0.0)
	fprintf(fd,"\033&a%dV",(int)(p->yoff * 720.0 / 25.4) );
  if (p->xoff != 0.0)
	fprintf(fd,"\033&a%dH",(int)(p->xoff * 720.0 / 25.4) );

/**
 ** Set Graphics Resolution (300 / 150 / 100 / 75):
 ** This is NO PCL level 3 feature, but LaserjetII and compatibles
 ** seem to accept it.
 **/
  fprintf(fd,"\033*t%dR", p->dpi_x);

/**
 ** Set Raster Width (in dots)
 **	Deskjet feature, good for saving internal memory!
 **/
  if (p->specials)
  {
	fprintf(fd,"\033*r%dS", width);
	switch (p->specials)
	{
	  case 4:	/* For future CMYK support	*/
	  case 3:	/* CMY				*/
		fprintf(fd,"\033*r-3U");
		break;
	  default:	/* Single color plane		*/
		fprintf(fd,"\033*r1U");
		break;
	}
  }

/**
 ** Start Raster Graphics at current position
 ** This is NO PCL level 3 feature, but LaserjetII and compatibles
 ** seem to accept it.
 **/
  fprintf(fd,"\033*r1A");
}




void	end_graphmode (FILE *fd)
{
/**
 ** End Raster Graphics
 **/
  fprintf(fd,"\033*rB");
}

